/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlBinop.h"
#include "HvSignalLine.h"
#include "HvControlIf.h"
#include "HvSignalVar.h"
#include "HvControlVar.h"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_GgCuqQ1c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_FDkzv40W_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ofkpe7Z7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_thEWKuum_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_JyN4ysF4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_a6AAMYhH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SRUoQLu6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_a2z48ytF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xb6axXVk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6ARykrvP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TuTZyPQx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_za68JdTi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VAGLOTGd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_f0pE0W8y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_wqIJVyfU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_qoDzuZII_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_QEM8sO5q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_vEwfMKUK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_XMOk8Zcb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MOdW3MIu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_NUO8GWUA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vZAGkNrl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JKv04Kiw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SEBDa2VP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_MGPveo1n_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fmQXKWTh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Wm9zIkm1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SWEiNgiV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_MOldP78E_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RR6LuYtL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_nPh1zkHa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_OJ8WR30p_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_uTEBwFxy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_sptmZspB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_eFwEqUa2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mjUMHTjM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VeWDdNYU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_L8qqvcE0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rh8rCZqo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GvOaYJOK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_glPjkGwg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_i7hcGk67_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MGOOSzl3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_x2sPtqPX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YFxye7aq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kThEVrxS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fvndMGCn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vbTzcGVe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_BWPEeKvb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7VakszQJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_G3cq2OtS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MAhIQZls_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_O62ppp1Q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tuZ5Qd3m_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pTeAeC7y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_dgfCHJDf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_65WweZQA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_GQAvUlTL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_dxGAwM04_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_oJ8aBEyC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_5PDFChbj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_8CfJ3vE6_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_9SlZkE1a;
  SignalEnvelope sEnv_vEwfMKUK;
  SignalEnvelope sEnv_XMOk8Zcb;
  SignalEnvelope sEnv_eFwEqUa2;
  SignalEnvelope sEnv_mjUMHTjM;
  SignalLine sLine_Db0yFUy8;
  SignalLine sLine_mWCR34e4;
  ControlBinop cBinop_GgCuqQ1c;
  ControlBinop cBinop_FDkzv40W;
  ControlBinop cBinop_Ofkpe7Z7;
  ControlBinop cBinop_thEWKuum;
  ControlBinop cBinop_SRUoQLu6;
  ControlBinop cBinop_xb6axXVk;
  ControlBinop cBinop_6ARykrvP;
  ControlBinop cBinop_TuTZyPQx;
  ControlVar cVar_za68JdTi;
  ControlVar cVar_f0pE0W8y;
  SignalVarf sVarf_uTBJTbkj;
  ControlVar cVar_wqIJVyfU;
  ControlIf cIf_qoDzuZII;
  ControlIf cIf_QEM8sO5q;
  ControlBinop cBinop_MOdW3MIu;
  ControlBinop cBinop_vZAGkNrl;
  ControlBinop cBinop_JKv04Kiw;
  ControlBinop cBinop_fmQXKWTh;
  ControlBinop cBinop_SWEiNgiV;
  SignalVarf sVarf_jo9F94Wa;
  ControlVar cVar_OJ8WR30p;
  ControlIf cIf_uTEBwFxy;
  ControlIf cIf_sptmZspB;
  ControlBinop cBinop_VeWDdNYU;
  ControlBinop cBinop_rh8rCZqo;
  ControlBinop cBinop_GvOaYJOK;
  ControlBinop cBinop_MGOOSzl3;
  ControlBinop cBinop_YFxye7aq;
  ControlVar cVar_BWPEeKvb;
  ControlBinop cBinop_7VakszQJ;
  SignalVarf sVarf_JlkZwRYL;
  SignalVarf sVarf_LNXIR76u;
  SignalVarf sVarf_LjoxaiQL;
  SignalVarf sVarf_2cfopEXD;
  ControlBinop cBinop_tuZ5Qd3m;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
